@@ -4,6 +4,7 @@ import importlib.machinery
 import importlib.util
 import logging
 import os
+import copy
 import sys
 import sysconfig
 import time
@@ -135,6 +136,12 @@ class NatlinkMain(metaclass=Singleton):
     def user(self, value: str):
         self.__user = value or ''
 
+    # QH added, for _control grammar of Unimacro:
+    def get_loaded_modules(self) -> Dict:
+        """return a copy of the loaded modules
+        """
+        return copy.copy(self.loaded_modules)
+
     # load_on_begin_utterance is a property...
     def get_load_on_begin_utterance(self) -> Any:
         """this value is most often True or False, taken from the config file
@@ -143,7 +150,8 @@ class NatlinkMain(metaclass=Singleton):
         the load_on_begin_utterance so many times. After these utterances,
         the value falls back to False.
         
-        With Vocola, there is one utterance delay in the updating of the changed vocola command files.
+        With Vocola, this value is set to 1, for a one time load_on_begin_utterance, wihtout the
+        need to toggle the microphone.
         """
         return self.__load_on_begin_utterance
 
@@ -153,6 +161,9 @@ class NatlinkMain(metaclass=Singleton):
         For Vocola, setting this value to 1 did not work, setting to 2 does, so
         you need one extra utterance for a new vocola command to come through.
         """
+        if isinstance(value, bool):
+            self.__load_on_begin_utterance = value
+            return
         if isinstance(value, int):
             if value > 0:
                 self.logger.info(f'set_load_on_begin_utterance to {value}')
@@ -160,12 +171,8 @@ class NatlinkMain(metaclass=Singleton):
             else:
                 self.logger.info('set_load_on_begin_utterance to False')
                 self.__load_on_begin_utterance = False
-        elif value in [True, False]:
-            self.__load_on_begin_utterance = value
-        else:
-            raise TypeError(f'set_load_on_begin_utterance, invalid type for value: {value} (type: {type(value)})')
-        
-
+            return
+        raise TypeError(f'set_load_on_begin_utterance, invalid type for value: {value} (type: {type(value)})')
     load_on_begin_utterance = property(get_load_on_begin_utterance, set_load_on_begin_utterance)
 
     # def _module_paths_in_dir(self, directory: str) -> List[Path]:
@@ -405,9 +412,10 @@ class NatlinkMain(metaclass=Singleton):
         elif self.load_on_begin_utterance:
             # manipulate this setting:
             value = self.load_on_begin_utterance
-            if isinstance(value, int):
+            if isinstance(value, bool):
+                pass
+            elif isinstance(value, int):
                 value -= 1
-                value = value or False
                 self.load_on_begin_utterance = value
             self.trigger_load()
                 
